2021-06-11 14:02:28 +02:00
From 0000000000000000000000000000000000000000 Mon Sep 17 00:00:00 2001
From: Phoenix616 <mail@moep.tv>
Date: Sat, 1 Feb 2020 16:50:39 +0100
Subject: [PATCH] Pillager patrol spawn settings and per player options
This adds config options for defining the spawn chance, spawn delay and
spawn start day as well as toggles for handling the spawn delay and
start day per player. (Based on the time played statistic)
When not per player it will use the Vanilla mechanic of one delay per
world and the world age for the start day.
diff --git a/src/main/java/net/minecraft/server/level/ServerPlayer.java b/src/main/java/net/minecraft/server/level/ServerPlayer.java
2024-12-03 19:21:12 +01:00
index a624cab05e9a4308a7f887f9edcf0ec1555f94a8..df409f2dc2c49282f128a9fdd9c248a2f6d9a7c9 100644
2021-06-11 14:02:28 +02:00
--- a/src/main/java/net/minecraft/server/level/ServerPlayer.java
+++ b/src/main/java/net/minecraft/server/level/ServerPlayer.java
2024-12-03 19:21:12 +01:00
@@ -295,6 +295,7 @@ public class ServerPlayer extends net.minecraft.world.entity.player.Player {
2021-06-11 14:02:28 +02:00
public boolean wonGame;
2024-01-24 11:45:17 +01:00
private int containerUpdateDelay; // Paper - Configurable container update tick rate
2024-01-21 13:56:22 +01:00
public long loginTime; // Paper - Replace OfflinePlayer#getLastPlayed
2024-01-21 12:11:43 +01:00
+ public int patrolSpawnDelay; // Paper - Pillager patrol spawn settings and per player options
2021-06-11 14:02:28 +02:00
// Paper start - cancellable death event
2024-01-24 11:45:17 +01:00
public boolean queueHealthUpdatePacket;
2021-06-11 14:02:28 +02:00
public net.minecraft.network.protocol.game.ClientboundSetHealthPacket queuedHealthUpdatePacket;
diff --git a/src/main/java/net/minecraft/world/level/levelgen/PatrolSpawner.java b/src/main/java/net/minecraft/world/level/levelgen/PatrolSpawner.java
2024-10-23 15:54:09 +02:00
index 8837db46ced2412942949a3c6f81ec7d9a0bacda..e93ef232b0426a1095dad05fc4acb2a74db5b689 100644
2021-06-11 14:02:28 +02:00
--- a/src/main/java/net/minecraft/world/level/levelgen/PatrolSpawner.java
+++ b/src/main/java/net/minecraft/world/level/levelgen/PatrolSpawner.java
2024-04-24 08:05:14 +02:00
@@ -24,7 +24,7 @@ public class PatrolSpawner implements CustomSpawner {
2021-06-11 14:02:28 +02:00
@Override
public int tick(ServerLevel world, boolean spawnMonsters, boolean spawnAnimals) {
2024-01-21 12:53:04 +01:00
- if (world.paperConfig().entities.behavior.pillagerPatrols.disable) return 0; // Paper - Add option to disable pillager patrols
+ if (world.paperConfig().entities.behavior.pillagerPatrols.disable || world.paperConfig().entities.behavior.pillagerPatrols.spawnChance == 0) return 0; // Paper - Add option to disable pillager patrols & Pillager patrol spawn settings and per player options
2021-06-11 14:02:28 +02:00
if (!spawnMonsters) {
return 0;
} else if (!world.getGameRules().getBoolean(GameRules.RULE_DO_PATROL_SPAWNING)) {
2024-04-24 08:05:14 +02:00
@@ -32,23 +32,51 @@ public class PatrolSpawner implements CustomSpawner {
2021-06-11 14:02:28 +02:00
} else {
2022-06-08 06:22:42 +02:00
RandomSource randomsource = world.random;
2021-06-11 14:02:28 +02:00
- --this.nextTick;
- if (this.nextTick > 0) {
2024-01-21 12:11:43 +01:00
+ // Paper start - Pillager patrol spawn settings and per player options
2021-06-11 14:02:28 +02:00
+ // Random player selection moved up for per player spawning and configuration
+ int j = world.players().size();
+ if (j < 1) {
2023-03-23 22:57:03 +01:00
return 0;
2021-06-11 14:02:28 +02:00
+ }
+
2022-06-08 06:22:42 +02:00
+ net.minecraft.server.level.ServerPlayer entityhuman = world.players().get(randomsource.nextInt(j));
2021-06-11 14:02:28 +02:00
+ if (entityhuman.isSpectator()) {
+ return 0;
+ }
+
+ int patrolSpawnDelay;
2022-06-09 10:51:45 +02:00
+ if (world.paperConfig().entities.behavior.pillagerPatrols.spawnDelay.perPlayer) {
2021-06-11 14:02:28 +02:00
+ --entityhuman.patrolSpawnDelay;
+ patrolSpawnDelay = entityhuman.patrolSpawnDelay;
2023-03-23 22:57:03 +01:00
} else {
- this.nextTick += 12000 + randomsource.nextInt(1200);
- long i = world.getDayTime() / 24000L;
2021-06-17 22:20:03 +02:00
+ this.nextTick--;
+ patrolSpawnDelay = this.nextTick;
2021-06-11 14:02:28 +02:00
+ }
+
+ if (patrolSpawnDelay > 0) {
2023-03-23 22:57:03 +01:00
+ return 0;
+ } else {
2021-06-11 14:02:28 +02:00
+ long days;
2022-06-09 10:51:45 +02:00
+ if (world.paperConfig().entities.behavior.pillagerPatrols.start.perPlayer) {
2021-11-24 11:04:30 +01:00
+ days = entityhuman.getStats().getValue(net.minecraft.stats.Stats.CUSTOM.get(net.minecraft.stats.Stats.PLAY_TIME)) / 24000L; // PLAY_ONE_MINUTE is actually counting in ticks, a misnomer by Mojang
2021-06-11 14:02:28 +02:00
+ } else {
+ days = world.getDayTime() / 24000L;
+ }
2022-06-09 10:51:45 +02:00
+ if (world.paperConfig().entities.behavior.pillagerPatrols.spawnDelay.perPlayer) {
+ entityhuman.patrolSpawnDelay += world.paperConfig().entities.behavior.pillagerPatrols.spawnDelay.ticks + randomsource.nextInt(1200);
2021-06-11 14:02:28 +02:00
+ } else {
2022-06-09 10:51:45 +02:00
+ this.nextTick += world.paperConfig().entities.behavior.pillagerPatrols.spawnDelay.ticks + randomsource.nextInt(1200);
2021-06-11 14:02:28 +02:00
+ }
- if (i >= 5L && world.isDay()) {
2022-06-08 06:22:42 +02:00
- if (randomsource.nextInt(5) != 0) {
2022-06-09 10:51:45 +02:00
+ if (days >= world.paperConfig().entities.behavior.pillagerPatrols.start.day && world.isDay()) {
+ if (randomsource.nextDouble() >= world.paperConfig().entities.behavior.pillagerPatrols.spawnChance) {
2024-01-21 12:11:43 +01:00
+ // Paper end - Pillager patrol spawn settings and per player options
2021-06-11 14:02:28 +02:00
return 0;
} else {
- int j = world.players().size();
if (j < 1) {
return 0;
} else {
2022-06-08 06:22:42 +02:00
- Player entityhuman = (Player) world.players().get(randomsource.nextInt(j));
2021-06-11 14:02:28 +02:00
if (entityhuman.isSpectator()) {
return 0;